%/***************************************************************************
% *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
% *   ghiathi@gmail.com                                                     *
% *                                                                         *
% *   This program is free software; you can redistribute it and/or modify  *
% *   it under the terms of the GNU General Public License as published by  *
% *   the Free Software Foundation; either version 3 of the License, or     *
% *   (at your option) any later version.                                   *
% *                                                                         *
% *   This program is distributed in the hope that it will be useful,       *
% *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
% *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
% *   GNU General Public License for more details.                          *
% *                                                                         *
% *   You should have received a copy of the GNU General Public License     *
% *   along with this program; if not, write to the                         *
% *   Free Software Foundation, Inc.,                                       *
% *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
% ***************************************************************************/


function  [tsterrMean tsterrStd tstTimeMean tstTimeStd trnerrMean trnerrStd trntimeMean ... 
           trntimeStd mCosineMean mCosineStd mGaussianMean mGaussianStd nsvMean nsvStd] = RunCombined( Path, dataset, logfilenamePrefix, method)

global trn tst model D2trn
global ExperimentRepeat 

tsterr (1:ExperimentRepeat) = inf;
trnerr (1:ExperimentRepeat) = inf;
trntime  (1:ExperimentRepeat) = inf;
tsttime  (1:ExperimentRepeat) = inf;
test_m_cosine   (1:ExperimentRepeat) = inf;
test_m_gaussian (1:ExperimentRepeat) = inf;
test_nsv (1:ExperimentRepeat) = inf;

for trntstitr=1:ExperimentRepeat
    sprintf ('%i',trntstitr);

    InitializeDataset_Stabilizer(Path, dataset, trntstitr);
        
    tic;
    if (strcmp(method, 'GaussianMixture') || strcmp(method, 'CosineAndGaussianMixture'))
    D2trn = zeros(trn.num_data,trn.num_data);            
        for k=1:trn.dim
            D2trn(:,:) = D2trn(:,:) + dist_euclid_ikl(trn.X(k,:)',trn.X(k,:)');
        end    
    end
    TrainCombinedKernel (dataset, trntstitr, logfilenamePrefix, method);
    trntime (trntstitr) = toc;

    if (model.m == 0 && size (model.options.sigmaWeights,1) == 0)
        break;
    end
    
    tic;
    ypred = svmclass(tst.X,model); % classify data
    tsttime (trntstitr) = toc;
    ypred = -2 * ypred + 3;
    tsterr(trntstitr) = cerror(ypred,tst.y); % compute error
    
    test_m_cosine   (trntstitr)   = model.m;
    test_m_gaussian (trntstitr)   = numel (model.options.sigmaWeights);
    test_nsv (trntstitr)          = model.nsv;

    ypred = svmclass(trn.X,model); % classify data
    ypred = -2 * ypred + 3;
    trnerr(trntstitr) = cerror(ypred,trn.y); % compute error
    
end


tsterrMean = mean (tsterr);
tsterrStd  = std (tsterr);
tstTimeMean = mean (tsttime);
tstTimeStd = std (tsttime);
trnerrMean = mean (trnerr);
trnerrStd = std (trnerr);
trntimeMean = mean(trntime);
trntimeStd = std(trntime);
mCosineMean = mean (test_m_cosine);
mCosineStd  = std (test_m_cosine);
mGaussianMean = mean (test_m_gaussian); 
mGaussianStd  = std (test_m_gaussian); 
nsvMean = mean (test_nsv);
nsvStd = std (test_nsv);

