%/***************************************************************************
% *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
% *   ghiathi@gmail.com                                                     *
% *                                                                         *
% *   This program is free software; you can redistribute it and/or modify  *
% *   it under the terms of the GNU General Public License as published by  *
% *   the Free Software Foundation; either version 3 of the License, or     *
% *   (at your option) any later version.                                   *
% *                                                                         *
% *   This program is distributed in the hope that it will be useful,       *
% *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
% *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
% *   GNU General Public License for more details.                          *
% *                                                                         *
% *   You should have received a copy of the GNU General Public License     *
% *   along with this program; if not, write to the                         *
% *   Free Software Foundation, Inc.,                                       *
% *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
% ***************************************************************************/

function [f] = SolveCombinedQCQP ()
    global  m trn model Gamma Gamma_norm InvSigma2 D2trn SVM_C
    persistent K 

    epsilon0 = 0.0001;
    model = [];
    K = ones(2*m,trn.num_data);
    for i=1:2*m
        activeidx = find(Gamma(floor((i+1) / 2),:));
        tG = Gamma(floor((i+1) / 2),activeidx);
        if (mod (i, 2) == 1)
            K(i,:) = trn.y(:)' .* cos (tG * trn.X(activeidx,:));
        else
            K(i,:) = trn.y(:)' .* sin (tG * trn.X(activeidx,:));
        end
    end

    %  $optimize (1-\epsilon_0)t + \epsilon_0\alpha^T\alpha - 2*\alpha^T e
    prob.c = [1-epsilon0 -2*ones(1,trn.num_data) zeros(1,2*m)];
    for i=1:trn.num_data
        prob.qosubi(i) = i + 1;
        prob.qosubj(i) = i + 1;
        prob.qoval (i) = 2 * epsilon0;
    end
    
    % $optimize t
    % prob.c = [1 zeros(1,trn.num_data) zeros(1,2*m)];

    % Next quadratic terms in the constraints.
    cCnstr = 3*m+3;
    cNewConstraints = size (InvSigma2);
    cVar   = trn.num_data + 2 * m + 1;
    prob.buc = zeros (1, cCnstr)';
    prob.blc = zeros (1, cCnstr)';

    % Constraint 1:   $\alpha^Ty  \eq 0
    k = 1;
    prob.afull(k,:) = [0 trn.y(:)' zeros(1,2*m)];
    prob.buc(k) = 0;
    prob.blc(k) = 0;

    t = 1;
    % Constraint 2:   $\alpha^T\alpha - t \leq 0
    k = k+1;
    for i=1:trn.num_data
        prob.qcsubk (t) = k;
        prob.qcsubi (t) = i+1;
        prob.qcsubj (t) = i+1;
        prob.qcval (t) = 2;  % since there is a one half implicit in Mosek formulation
        t = t + 1;
    end

    prob.afull(k,1) = -1;
    prob.blc(k) = -inf;
    prob.buc(k) = 0;    % 0 if Identity kernel is active, inf otherwise

    % Constraints 3-m+2:  $u_g^2 + v_g^2 - t \leq 1\frac{D^2}*norm(\gamma)^2$
    for i=1:m
        k = k+1;
        prob.qcsubk (t) = k;
        prob.qcsubi (t) = 1 + trn.num_data + i;
        prob.qcsubj (t) = 1 + trn.num_data + i;
        norm_a = norm (Gamma(i,:), Gamma_norm);
        prob.qcval (t) = 2 / Stabilizer (norm_a);
        t = t+1;

        prob.qcsubk (t) = k;
        prob.qcsubi (t) = 1 + trn.num_data + m + i;
        prob.qcsubj (t) = 1 + trn.num_data + m + i;
        prob.qcval (t) = 2 / Stabilizer (norm_a);
        t = t+1;

        prob.afull(k,1) = -1;

        prob.buc(k) = 0;
        prob.blc(k) = -inf;
    end

    % Constraints m+3-2m+2:
    for i=1:m
        k = k+1;
        prob.afull(k,2:2+trn.num_data-1) = K(2*i-1,:);
        prob.afull(k,1+trn.num_data+i) = -1;
        prob.buc(k) = 0;
        prob.blc(k) = 0;
    end

    % Constraints 2m+3-3m+2:
    for i=1:m
        k = k+1;
        prob.afull(k,2:2+trn.num_data-1) = K(2*i,:);
        prob.afull(k,1+trn.num_data+m+i) = -1;
        prob.buc(k) = 0;
        prob.blc(k) = 0;
    end
    
    % Constraints 3m+3:  $\alpha^Te=2
    k = k+1;
    prob.afull(k,:) = [0 ones(1,trn.num_data) zeros(1,2*m)];
    prob.buc(k) = 2;
    prob.blc(k) = 2;

    % Constraints 3m+3-3m+3+size (InvSigma2):
    for s=1:size(InvSigma2)
        k = k+1;        
        Kk=0;
        Kk = Kk + InvSigma2(s) * D2trn(:,:);
        
        Kk = exp (-Kk);
        Kk = Kk .* (trn.y * trn.y');
        
        %norm_Sigma = norm (InvSigma2(s,:));
        norm_Sigma = norm (InvSigma2(s));
        stabilizer_coef = StabilizerInvSigma (norm_Sigma);
        
        L = tril((2 * stabilizer_coef) * Kk);
        count_nonzero = nnz(L);
        prob.qcsubk(t:t+count_nonzero-1) = k;
        [prob.qcsubj(t:t+count_nonzero-1) prob.qcsubi(t:t+count_nonzero-1) prob.qcval(t:t+count_nonzero-1)] = find (L');
        prob.qcsubi(t:t+count_nonzero-1) = prob.qcsubi(t:t+count_nonzero-1) + 1;
        prob.qcsubj(t:t+count_nonzero-1) = prob.qcsubj(t:t+count_nonzero-1) + 1;
        t = t + count_nonzero;
        
%         for i=1:trn.num_data
%             for j=1:i
%                 [i j k]
%                 [prob.qcsubk(t) prob.qcsubi(t) prob.qcsubj(t) prob.qcval(t)]
%                 assert (prob.qcsubk (t) == k);
%                 assert (prob.qcsubi (t) == i);
%                 assert (prob.qcsubj (t) == j);
%                 assert (prob.qcval (t) == 2 * Kk(i,j));
%                 t = t + 1;
%             end
%         end
        
%         for i=1:trn.num_data
%                 prob.qcsubk (t:t+i-1) = k * ones (1,i);
%                 prob.qcsubi (t:t+i-1) = (i+1) * ones (1,i);
%                 prob.qcsubj (t:t+i-1) = 2:i+1;
%                 prob.qcval (t:t+i-1) = 2 * Kk(i,1:i);
%                 t = t + i;
%         end
        prob.afull(k,1) = -1;
        prob.blc(k) = -inf;
        prob.buc(k) = 0;
    end 
    
    prob.a = sparse (prob.afull);
    prob.blx = [0   zeros(1,trn.num_data) -inf * ones(1, 2*m)];
    %prob.bux = [];
    prob.bux = [inf  SVM_C * ones(1, trn.num_data)  inf * ones(1, 2*m ) ];

    mosekparms = [];
    % Primal feasibility tolerance for the primal solution
    mosekparms.MSK_DPAR_INTPNT_CO_TOL_PFEAS = 1.0e-12;
    % Dual feasibility tolerance for the dual solution
    mosekparms.MSK_DPAR_INTPNT_CO_TOL_DFEAS = 1.0e-12;
    % Relative primal-dual gap tolerance.
    mosekparms.MSK_DPAR_INTPNT_CO_TOL_REL_GAP = 1.0e-12;
    mosekparms.MSK_IPAR_INTPNT_MAX_ITERATIONS = 1000;
    mosekparms.MSK_DPAR_INTPNT_CO_TOL_NEAR_REL = 1;
    mosekparms.MSK_DPAR_INTPNT_NL_TOL_NEAR_REL = 1;
    mosekparms.MSK_IPAR_BI_IGNORE_NUM_ERROR = 1;
    %[ret,res] = mosekopt ('minimize echo(0)', prob, mosekparms );
    [ret,res] = mosekopt ('minimize echo(0) statuskeys(1)', prob, mosekparms );
    if (ret ~= 0)
        disp (sprintf ('mosekopt returned error %d.', ret));
    end
    %res.sol.itr.xx(1)
    %sum (res.sol.itr.y(2:2+m))
    %activeidx = find (res.sol.itr.xc(3:m+2) > -inf);
    activeidx = find (res.sol.itr.skc (3:m+2) == 4);
    sigmaActiveidx = find (res.sol.itr.skc (3*m+4:3*m+3+size(InvSigma2)) == 4);

%     temp0     = size (find (res.sol.itr.skc (3:m+2) == 0),1);       
%     temp1     = size (find (res.sol.itr.skc (3:m+2) == 1),1);
%     temp2     = size (find (res.sol.itr.skc (3:m+2) == 2),1);
%     temp3     = size (find (res.sol.itr.skc (3:m+2) == 3),1);    
%     temp4     = size (find (res.sol.itr.skc (3:m+2) == 4),1);    
%     temp5     = size (find (res.sol.itr.skc (3:m+2) == 5),1);        
%     disp ([temp0, temp1, temp2, temp3, temp4, temp5]);
% 
%     if (temp0 + temp1 + temp3 + temp5 > 0)
%         stop = 1;
%     end
    
    f     = -res.sol.itr.xx(1);
    alpha = res.sol.itr.xx(2:trn.num_data+1)';

    alphaidx = find (alpha > 1*epsm);
    %alphaidx = find (alpha > -inf);    
    model.options.ker = 'CosineMixture';
    model.options.arg = Gamma (activeidx,:)'; % kernel argument
    model.options.gammaWeights  = -res.sol.itr.y(2 + activeidx)';
    
    model.options.sigmaArg = InvSigma2 (sigmaActiveidx);     % kernel argument
    model.options.sigmaWeights  = -res.sol.itr.y(3*m+3 + sigmaActiveidx)';   % kernel weights
  
    model.alpha = alpha;
    model.Alpha = alpha (alphaidx)' .* trn.y(alphaidx);
    model.nsv = size(model.Alpha, 1);
    model.sv.X = trn.X(:,alphaidx);
    model.sv.y = (trn.y(alphaidx)+3) / 2;
    model.sv.inx = alphaidx';
    model.options.C = -1 / res.sol.itr.y(2);
    model.options.tau = -res.sol.itr.y(2);
    
    %         den = sum (alpha) / 2;
    %         alpha = alpha / den;
    %         model.b = model.b / den;
    %         model.Alpha = model.Alpha / den;
                
    originalKernelIndex = zeros (numel(activeidx),1);
    for d=1:m
        if (find (activeidx == d))
            originalKernelIndex (find (activeidx == d)) = d;
        end
    end

    model.m = numel(activeidx);
    m = model.m;

%     originalKernelIndexOld = originalKernelIndex
%     activeidxOld = activeidx
%     gamaWeightsOld = model.options.gammaWeights
%     mOld = m
    
    
    for i=1:m
            norm_a = norm (Gamma(originalKernelIndex(i),:), Gamma_norm);
            model.options.rho (i) = sqrt(model.options.gammaWeights (i));
            model.options.gammaWeights(i) = model.options.gammaWeights(i) / Stabilizer(norm_a);
    end
    
    for i=1:numel (model.options.sigmaWeights)
            norm_Sigma = norm (model.options.sigmaArg(i,1));
            stabilizer_coef = StabilizerInvSigma (norm_Sigma);
            model.options.sigmaWeights(i) = model.options.sigmaWeights(i) * stabilizer_coef;
    end
       
        
    for i=1:m
            if (m > numel (originalKernelIndex))
                originalKernelIndex
                activeidx
                model.options.gammaWeights
                m
                m = m;
                disp ('Ghiasi: it is a known bug. I could not understand why in some runs m > numel(originalKernelIndex)')
                disp ('If you could solve it, please notify me too.')
            end
            model.CosineCoef(i) = alpha (alphaidx) * K(2*originalKernelIndex(i)-1,alphaidx)' * model.options.gammaWeights(i);
            model.SineCoef(i)   = alpha (alphaidx) * K(2*originalKernelIndex(i)  ,alphaidx)' * model.options.gammaWeights(i);
    end
    
    trn.sample_weights = model.Alpha';
    trn.active_samples = alphaidx;
 
    Gamma = model.options.arg';
    InvSigma2 = model.options.sigmaArg; 
    
    [tmp positiveSV] = max (model.Alpha(:));
    [tmp negativeSV] = min (model.Alpha(:));
    ipos = alphaidx(positiveSV);
    ineg = alphaidx(negativeSV);
        
    margin = model.Alpha (positiveSV) / model.options.C - model.Alpha (negativeSV) / model.options.C;
    for i=1:m
            margin = margin + trn.y(ipos)   * K(2*originalKernelIndex(i),ipos) * alpha (alphaidx) ...
                                            * K(2*originalKernelIndex(i),alphaidx)' * model.options.gammaWeights (i);
            margin = margin + trn.y(ipos)   * K(2*originalKernelIndex(i)-1,ipos) * alpha (alphaidx) ...
                                            * K(2*originalKernelIndex(i)-1,alphaidx)' * model.options.gammaWeights (i);                                        

            margin = margin - trn.y(ineg)   * K(2*originalKernelIndex(i),ineg) * alpha (alphaidx) ...
                                            * K(2*originalKernelIndex(i),alphaidx)' * model.options.gammaWeights (i);
            margin = margin - trn.y(ineg)   * K(2*originalKernelIndex(i)-1,ineg) * alpha (alphaidx) ...
                                            * K(2*originalKernelIndex(i)-1,alphaidx)' * model.options.gammaWeights (i);                                        
    end

    for i=1:size(model.options.sigmaArg,1)
            for j=1:model.nsv
                k = alphaidx(j);
                margin = margin + model.Alpha (j) * model.options.sigmaWeights (i) * ...
                                exp (-sum(model.options.sigmaArg(i,:)' .* (trn.X(:,k) - trn.X(:,ipos)).^2));
                margin = margin - model.Alpha (j) * model.options.sigmaWeights (i) * ...
                                exp (-sum(model.options.sigmaArg(i,:)' .* (trn.X(:,k) - trn.X(:,ineg)).^2));
            end
    end

    
    bigalphaidx = find (alpha > 0.001);
    
    for j=1:min (size(bigalphaidx,2), 10)
        u = bigalphaidx(j);
        b(j) = margin * trn.y(u) / 2 - trn.y (u) * model.alpha (u) / model.options.C;
        for i=1:m
                b(j) = b(j) - trn.y(u) * K(2*originalKernelIndex(i)-1,u) * alpha (alphaidx) ... 
                                       * K(2*originalKernelIndex(i)-1,alphaidx)' * model.options.gammaWeights (i);
                b(j) = b(j) - trn.y(u) * K(2*originalKernelIndex(i),u) * alpha (alphaidx) ... 
                                       * K(2*originalKernelIndex(i),alphaidx)' * model.options.gammaWeights (i);                                   
        end
        
        for i=1:size(model.options.sigmaArg,1)
            for k=1:model.nsv
                v = alphaidx(k);
                b(j) = b(j) - model.Alpha (k) * model.options.sigmaWeights (i) * ...
                                exp (-sum(model.options.sigmaArg(i,:)' .* (trn.X(:,u) - trn.X(:,v)).^2));
            end
        end
    end

    model.b = mean(b);
    
end
