%/***************************************************************************
% *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
% *   ghiathi@gmail.com                                                     *
% *                                                                         *
% *   This program is free software; you can redistribute it and/or modify  *
% *   it under the terms of the GNU General Public License as published by  *
% *   the Free Software Foundation; either version 3 of the License, or     *
% *   (at your option) any later version.                                   *
% *                                                                         *
% *   This program is distributed in the hope that it will be useful,       *
% *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
% *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
% *   GNU General Public License for more details.                          *
% *                                                                         *
% *   You should have received a copy of the GNU General Public License     *
% *   along with this program; if not, write to the                         *
% *   Free Software Foundation, Inc.,                                       *
% *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
% ***************************************************************************/


function TestUSPS ()

global SVM_C Beta Gamma_norm lmbfgs_max_iters lmbfgs_M 
global ExperimentRepeat cTotalIter cRepeated maxIter1 epsilon_fine
global trn tst model
global lmbfgs_max_iters_inner logfilename

    N = 7291;
    dataset = 'usps_0-4,5-9';
    Path = '/home/user/work/Datasets';
    %Path = 'C:\Program Files\MATLAB\R2008a\work\Datasets';

    ExperimentRepeat = 1;
    cTotalIter = 500;
    cRepeated  = 4;
    maxIter1  = 1000;
    epsilon_fine  = 0.001;
    lmbfgs_max_iters = 100;
    lmbfgs_max_iters_inner  = -1; %LocallyAdaptLBFGS is disabled
    lmbfgs_M = 17;
    Gamma_norm = 2;  % 1 or 2
    CVTime = inf;
    
    SVM_C = 3; % SVM_C>=2 means infinity
    Beta = 3.0;
    logfilenamePrefix = '';  
    method = 'CosineMixture';
        
    sprintf ('CV Finished with Beta=%f. Now we do testing', Beta)
      
    %           Perform Real Test with Chosen Beta
    tsterr (1:ExperimentRepeat) = inf;
    trnerr (1:ExperimentRepeat) = inf;
    trntime  (1:ExperimentRepeat) = inf;
    tsttime  (1:ExperimentRepeat) = inf;
    test_m   (1:ExperimentRepeat) = inf;
    test_nsv (1:ExperimentRepeat) = inf;
    
    for i=1:ExperimentRepeat
        InitializeDataset_Stabilizer(Path, dataset, 1);
       
        trn.X = trn.X(:,1:N);
        trn.y = trn.y(1:N);
        trn.num_pos_data = size(find(trn.y ==1),1);
        trn.num_neg_data = size(find(trn.y ==-1),1);
        trn.num_data = N;
        trn.sample_weights = zeros(1,trn.num_data);
        trn.active_samples = 1:trn.num_data;    
        for j=1:trn.num_data
            if (trn.y(j) == 1)
                trn.sample_weights(j) = 1/trn.num_pos_data;
            else
                trn.sample_weights(j) = -1/trn.num_neg_data;
            end
        end

        logfilename = sprintf ('usps.txt');        
        tic;
        TrainCombinedKernel (dataset, 1, logfilenamePrefix, method); 
        trntime (i) = toc;

        tic;
        ypred = svmclass(tst.X,model); % classify data
        tsttime (i) = toc;

        ypred = -2 * ypred + 3;
        tsterr(i) = cerror(ypred,tst.y); % compute error

        test_m   (i)   = model.m;
        test_nsv (i)   = model.nsv;

        ypred = svmclass(trn.X,model); % classify data
        ypred = -2 * ypred + 3;
        trnerr(i) = cerror(ypred,trn.y); % compute error
    end

    tsterrMean = mean (tsterr);
    tsterrStd  = std (tsterr);
    tstTimeMean = mean (tsttime);
    tstTimeStd = std (tsttime);
    trnerrMean = mean (trnerr);
    trnerrStd = std (trnerr);
    trntimeMean = mean(trntime);
    trntimeStd = std(trntime);
    mMean = mean (test_m);
    mStd  = std (test_m);
    nsvMean = mean (test_nsv);
    nsvStd = std (test_nsv);

    out = sprintf ('dataset=%s\n Beta=%f\n CVTime=%f\n tsterrMean=%f\n tsterrStd=%f\n tstTimeMean%f\n tstTimeStd=%f\n trnerrMean=%f\n trnerrStd=%f\n trntimeMean=%f\n trntimeStd=%f\n mMean=%f\n mStd=%f\n nsvMean=%f\n nsvStd=%f\n \n', ...
        dataset, Beta, CVTime, tsterrMean ,tsterrStd ,tstTimeMean ,tstTimeStd ,trnerrMean ,trnerrStd ,trntimeMean ,trntimeStd ,mMean ,mStd ,nsvMean ,nsvStd);
    
    display (out);
    
    inParams = sprintf ('ExperimentRepeat = %f\ncTotalIter = %f \ncRepeated  = %f\nmaxIter1 = %f \nmaxIter2 = %f\nepsilon_gradient  = %f\nepsilon_fine\n lmbfgs_max_iters = %f\nlmbfgs_M = %f\nGamma_norm = %f\n' ...
        ,ExperimentRepeat, cTotalIter,cRepeated ,maxIter1, maxIter2, epsilon_gradient ,epsilon_fine ,lmbfgs_max_iters,lmbfgs_M, Gamma_norm);
    fid = fopen(strcat(dataset , date),'wt');
    fwrite(fid,inParams);
    fwrite(fid,out);   
    fclose(fid);
