%/***************************************************************************
% *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
% *   ghiathi@gmail.com                                                     *
% *                                                                         *
% *   This program is free software; you can redistribute it and/or modify  *
% *   it under the terms of the GNU General Public License as published by  *
% *   the Free Software Foundation; either version 3 of the License, or     *
% *   (at your option) any later version.                                   *
% *                                                                         *
% *   This program is distributed in the hope that it will be useful,       *
% *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
% *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
% *   GNU General Public License for more details.                          *
% *                                                                         *
% *   You should have received a copy of the GNU General Public License     *
% *   along with this program; if not, write to the                         *
% *   Free Software Foundation, Inc.,                                       *
% *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
% ***************************************************************************/

function [bestVal fTotal] = TrainCombinedKernel (dataset, trntstitr, logfilenamePrefix, method)
        global cTotalIter cRepeated maxIter1 epsilon_fine 
        global trn tst model Gamma InvSigma2 startPoints Gamma_norm Beta SVM_C m cteta steta
        global cTotalIterGaussian cRepeatedGaussian
        
        if (isempty(logfilenamePrefix))
                EnableLogFile = false;
        else
                EnableLogFile = true;
        end
        
        SVM_C = max (SVM_C, max (1 / trn.num_pos_data, 1 / trn.num_neg_data));
      
        Gamma = [];
        startPoints = [];
        InvSigma2 = [];
        bestSigmaVal = 0;
        bestGammaVal = 0;
       
        bestUptoNow = 1 / trn.num_neg_data + 1 / trn.num_pos_data;
        fTotalPrev1 = inf;
        cteta = zeros(1, trn.num_data);
        steta = zeros(1, trn.num_data);
        %weights = [1];  % weight for delta kernel
        
        model.options.rho = [];
        model.options.tau = 1;        
        model.m = 0;
        m = 0;
        
        if EnableLogFile
         fid = fopen(sprintf('%s_%s-%2.2f-%2.2f-%i',logfilenamePrefix, dataset, Beta, SVM_C,trntstitr),'wt');
        end
        
        prevNumIters = 1;
        for iter1=1:maxIter1
                  
            % Perform Local adaptation only for cosine kernels
            if (iter1 > 1 && model.m > 0)  %&& prevNumIters > 0
                prevNumIters = LocallyAdaptLBFGS(iter1);
            end
           
            prevGamma = Gamma;
            prevInvSigma2 = InvSigma2;

            if (strcmp(method, 'CosineMixture') || strcmp(method, 'CosineAndGaussianMixture'))
                disp ('Begin FindBestGammas_Stabilizer');
                t_begin = clock;
                bestGammaVal = FindBestGammas_Stabilizer(cTotalIter, cRepeated, bestUptoNow);
                t_end = clock;
                disp(sprintf ('End FindBestGammas_Stabilizer t= %i with %i points', etime (t_end, t_begin), size(Gamma,1)));

                m = size(prevGamma,1) + size(Gamma,1);
                m = min (500, m);
                m = m - size(prevGamma,1);
                Gamma = [prevGamma; Gamma(1:m,:)];
                Gamma = round(Gamma * 10000) / 10000;
                Gamma = unique(Gamma,'rows');
                m = size(Gamma,1);
            end

            if (strcmp(method, 'GaussianMixture') || strcmp(method, 'CosineAndGaussianMixture'))
                disp ('Begin FindBestInvSigma2s');
                t_begin = clock;
                bestSigmaVal = FindBestInvSigma2sIsotropic(cTotalIterGaussian, cRepeatedGaussian, bestUptoNow); %bestUptoNow
                t_end = clock;
                disp(sprintf ('End FindBestInvSigma2s t= %i with %i points', etime (t_end, t_begin), size(InvSigma2,1)));

                InvSigma2 = [prevInvSigma2; InvSigma2];
                InvSigma2 = round(InvSigma2 * 10000) / 10000;            
                InvSigma2 = unique(InvSigma2,'rows');
            end
                

            if (m == 0 && size (InvSigma2,1) == 0)
                bestVal = inf;
                if EnableLogFile
                    fclose (fid);
                end
                return;
            end
            
            disp ('Begin SolveCombinedQCQP');
            t_begin = clock;
            fTotal = SolveCombinedQCQP ();
            t_end = clock;
            disp (sprintf ('End SolveCombinedQCQP t= %i', etime (t_end, t_begin)));
            disp (sprintf ('fCorrected = %f, f = %f, Tau = %f\n', fTotal, fTotal, model.options.tau));

            if (model.m == 0  && size (InvSigma2,1) == 0)
                bestVal = inf;
                if EnableLogFile
                     fclose (fid);
                end
                return;
            end
            
            ypred = svmclass(trn.X,model); % classify data
            ypred = -2 * ypred + 3;
            trn_err = cerror(ypred,trn.y); % compute error
            
            ypred = svmclass(tst.X,model); % classify data
            ypred = -2 * ypred + 3;
            tst_err = cerror(ypred,tst.y); % compute error

            if (strcmp(method, 'CosineMixture'))
                disp ([iter1 bestGammaVal fTotal model.m model.nsv/100 trn_err*100 tst_err*100]);
            elseif (strcmp(method, 'GaussianMixture'))
                disp ([iter1 bestSigmaVal fTotal size(model.options.sigmaArg,1) model.nsv/100 trn_err*100 tst_err*100]);
            elseif (strcmp(method, 'CosineAndGaussianMixture'))
                disp ([iter1 bestGammaVal bestSigmaVal fTotal model.m size(model.options.sigmaArg,1) model.nsv/100 trn_err*100 tst_err*100]);%                    
            end
            
            bestUptoNow = -fTotal;           

            if (fTotalPrev1 - fTotal < -epsilon_fine * fTotalPrev1)
                break;
            end

            if EnableLogFile
                fprintf(fid,'\niter1=%i, bestVal=%8.8f, fTotal=%8.8f, model.m=%i, model.nsv=%i, trn_err=%8.4f, tst_err==%8.8f\t', ...
                    iter1, bestGammaVal, fTotal, model.m, model.nsv, trn_err, tst_err);
            end
            
            fTotalPrev1 = fTotal;
        end
        
       if EnableLogFile
           fclose (fid);
       end
       bestVal = min (bestGammaVal, bestSigmaVal);
end

