/****************************************************************************
 *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
 *   ghiathi@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mex.h"
#include "matrix.h"
#include <math.h>
#include <stdlib.h>
#include <fstream>
#include "lbfgs.h"

long	n, N, NN;
double  *maxiters;
double *Ds;
double *alpha;
double *x0;

static int progress(
    void *instance,
    const lbfgsfloatval_t *x,
    const lbfgsfloatval_t *g,
    const lbfgsfloatval_t fx,
    const lbfgsfloatval_t xnorm,
    const lbfgsfloatval_t gnorm,
    const lbfgsfloatval_t step,
    int n,
    int k,
    int ls
    )
{
	//mexPrintf("Iteration: %d\t", k);
    //mexPrintf("  fx = %f, x[0] = %f, x[1] = %f\t", fx, x[0], x[1]);
    //mexPrintf("  xnorm = %f, gnorm = %f, step = %f\n", xnorm, gnorm, step);
    return 0;
}


static lbfgsfloatval_t evaluate(void *instance, const lbfgsfloatval_t *rho,  lbfgsfloatval_t *g,
    const int n, const lbfgsfloatval_t step )
{
        double f = 0;
        double *sigma = new double [n];
        
        {
        for (unsigned int k=0 ; k<n ; ++k)
                {
                g[k] = 0;
                sigma[k] = rho[k] * rho[k];
                }
        }

        for (unsigned int i=0; i<N ; ++i)
        {
	        for (unsigned int j=i,jN=i*N; j<N ; ++j,jN+=N)
	        {
		        double d = 0.0;
		        const unsigned int ijN = i+jN;
	
		        for (unsigned int k=0 ; k<n; ++k)
			        d += sigma[k] * Ds[ijN+k*NN];

		        const double w = alpha[i]*alpha[j]*exp(-d);

		        f -= w;

		        for (unsigned int k=0 ; k<n ; ++k)
		        {
			        const double tmp = w*Ds[ijN+k*NN];
			        g[k] += tmp;
		        }
	        }
        }
/*                
        for (unsigned int i=0; i<N-1 ; ++i)
        {
	        for (unsigned int j=i+1,jN=(i+1)*N; j<N ; ++j,jN+=N)
	        {
		        double d = 0.0;
		        const unsigned int ijN = i+jN;
	
		        for (unsigned int k=0 ; k<n; ++k)
			        d += sigma[k] * Ds[ijN+k*NN];

		        const double w = alpha[i]*alpha[j]*exp(-d);

		        f -= w;

		        for (unsigned int k=0 ; k<n ; ++k)
		        {
			        const double tmp = w*Ds[ijN+k*NN];
			        g[k] += tmp;
		        }
	        }
        }
*/

        for (unsigned int k=0 ; k<n ; ++k)
                {
                g[k] *= 2 * rho[k];
                }

/*
	double alpha2 = 0.0;
	for (unsigned int i=0 ; i<N ; ++i)
		alpha2 += alpha[i]*alpha[i];

	f -= 0.5*alpha2;
*/
	
	delete[] sigma;

return f;
}


/* ==============================================================
 Main MEX function - interface to Matlab.
============================================================== */
void mexFunction( int nlhs, mxArray *plhs[],
		  int nrhs, const mxArray *prhs[] )
{
        long	        i, j, ret;
        int		M;
        double          epsg = 1e-6;
        double          epsf = 1e-6;
        double          epsx = 1e-6;
        int		info;
        double          *Sigma, *fval;
        lbfgsfloatval_t fx;
        lbfgs_parameter_t param;

   /* K = kernel( data, ker, arg ) */
   /* ------------------------------------------- */
   if( nrhs == 5)
   {
      if( !mxIsNumeric(prhs[0]) || !mxIsDouble(prhs[0]) ||
        mxIsEmpty(prhs[0])    || mxIsComplex(prhs[0]) )
        mexErrMsgTxt("Matrix Dtrn must be a real matrix.");

      if( !mxIsNumeric(prhs[1]) || !mxIsDouble(prhs[1]) ||
        mxIsEmpty(prhs[1])    || mxIsComplex(prhs[1]) )
        mexErrMsgTxt("Sigma must be a real matrix.");

      if( !mxIsNumeric(prhs[2]) || !mxIsDouble(prhs[2]) ||
        mxIsEmpty(prhs[2])    || mxIsComplex(prhs[2]) )
        mexErrMsgTxt("vector alpha must be a real matrix.");


      /* get pointer at input vectors */

        n = mxGetM(prhs[1]);
        N = mxGetM(prhs[2]);
        NN = N * N;
        
        {
	const unsigned int nDims = mxGetNumberOfDimensions(prhs[0]);
	const mwSize *dims = mxGetDimensions(prhs[0]);
	if ((N != dims[1]) || (N!=dims[0]))
		mexErrMsgTxt("First argument must be symmetric in first 2 dimensions");
        }
       
	if (mxGetN(prhs[1]) != 1)
		mexErrMsgTxt("Second argument must be of size [Nfactors(n),1]");

	if (mxGetM(prhs[1]) != n)
		mexErrMsgTxt("Second argument must be of size [Nfactors,1]");
	
	if (mxGetN(prhs[1]) != 1)
		mexErrMsgTxt("Second argument must be of size [Nfactors,1]");

	if (mxGetN(prhs[2]) != 1)
		mexErrMsgTxt("Third argument must be of size [Npoints,1]");


        Ds = mxGetPr(prhs[0]);
        x0 = mxGetPr(prhs[1]);        
        alpha = mxGetPr(prhs[2]);

        if (mxGetM(prhs[3]) != 1)
                mexErrMsgTxt("mxGetM(prhs[3]) != 1");
        if (mxGetN(prhs[3]) != 1)
                mexErrMsgTxt("mxGetN(prhs[3]) != 1");
          maxiters = mxGetPr(prhs[3]);         

        if (mxGetM(prhs[4]) != 1)
                mexErrMsgTxt("mxGetM(prhs[4]) != 1");
        if (mxGetN(prhs[4]) != 1)
                mexErrMsgTxt("mxGetN(prhs[4]) != 1");
        M = (int) (*mxGetPr(prhs[4]));
        if (n < M)
                M = n;


                /* creates output vector f. */
                plhs[0] = mxCreateDoubleMatrix(n, 1, mxREAL);
                Sigma   = mxGetPr(plhs[0]);

		/* Initialize the parameters for the L-BFGS optimization. */
		lbfgs_parameter_init(&param);
		param.m = M;
		param.linesearch     = LBFGS_LINESEARCH_BACKTRACKING;
		param.max_linesearch = 20;
		param.max_iterations = *maxiters;
		param.epsilon        = 10e-6;


		lbfgsfloatval_t *lbfgs_x0 = lbfgs_malloc (n);
		for (i=0; i < n; i++)
			lbfgs_x0[i] = sqrt (fabs (x0[i])); //fabs is not necessary

		/*
			Start the L-BFGS optimization; this will invoke the callback functions
			evaluate() and progress() when necessary.
		 */

    	        ret = lbfgs(n, lbfgs_x0, &fx, evaluate, progress, NULL, &param);

		/* Report the result. */
		if (ret!=0 && ret != 1  && ret != 2 && ret != -999)
		        mexPrintf("L-BFGS optimization terminated with status code = %d\n", ret);

		for (i=0; i < n; i++)
			Sigma[i] = lbfgs_x0[i] * lbfgs_x0[i];

		if (nlhs > 1)
		{
			/* creates output vector Sigma. */
			plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
			fval    = mxGetPr(plhs[1]);
			*fval	= fx;
		}

		lbfgs_free(lbfgs_x0);
   }
   else
   {
      mexErrMsgTxt("Wrong number of input arguments.");
   }

   return;
}
