/****************************************************************************
 *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
 *   ghiathi@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "mex.h"
#include "matrix.h"
#include <math.h>
#include <stdlib.h>
#include <fstream>
#include "lbfgs.h"

long	N, NN;
double  *maxiters;
double *Ds;
double *alpha;
double *x0;
double Eta;
lbfgsfloatval_t *lbfgs_x0;

static int progress(
    void *instance,
    const lbfgsfloatval_t *x,
    const lbfgsfloatval_t *g,
    const lbfgsfloatval_t fx,
    const lbfgsfloatval_t xnorm,
    const lbfgsfloatval_t gnorm,
    const lbfgsfloatval_t step,
    int n,
    int k,
    int ls
    )
{
    //mexPrintf("Iteration: %d\t", k);
    //mexPrintf("  fx = %f, x[0] = %f\t", fx, x[0]);
    //mexPrintf("  xnorm = %f, gnorm = %f, step = %f\n", xnorm, gnorm, step);
    return 0;
}


static lbfgsfloatval_t evaluate(void *instance, const lbfgsfloatval_t *rho,  lbfgsfloatval_t *g,
    const int n, const lbfgsfloatval_t step )
{
        double fMain = 0;
        double fReg  = 0; 
        double f     = 0;
        double sigma;
        
        g[0] = 0;
        sigma = rho[0] * rho[0];

        for (unsigned int i=0; i<N ; ++i)
        {
	        for (unsigned int j=(i+1),jN=(i+1)*N; j<N ; ++j,jN+=N)
	        {
		        double d = 0.0;
		        const unsigned int ijN = i+jN;
	
        	        d = sigma * Ds[ijN];

		        const double w = alpha[i]*alpha[j]*exp(-d);

		        fMain -= w;

		        g[0] += w*Ds[ijN];
	        }
        }
        
        fMain *= 2;

        for (unsigned int i=0; i<N ; ++i)
                fMain -= alpha[i]*alpha[i];
        
        fReg = exp (-Eta * sigma * sigma);
        f = fMain * fReg;
        
        g[0] *= 2 * rho[0];
        
        g[0] = g[0] * fReg - 2 * Eta * sigma * f;

        //mexPrintf("  f = %f, sigma = %f\t true_n=%d\t Eta = %f\t fMain=%f\t fReg=%f \n", f, sigma,true_n, Eta, fMain, fReg);
        
        //mexPrintf(" lbfgs_x0 = %p\t rho = %p\tlbfgs_x0[0] = %f\t rho[0] = %f\n" , lbfgs_x0, rho, lbfgs_x0[0], rho[0]);
return f;
}


/* ==============================================================
 Main MEX function - interface to Matlab.
============================================================== */
void mexFunction( int nlhs, mxArray *plhs[],
		  int nrhs, const mxArray *prhs[] )
{
        long	        i, j, ret;
        int		M;
        double          epsg = 1e-6;
        double          epsf = 1e-6;
        double          epsx = 1e-6;
        int		info;
        double          *Sigma, *fval;
        lbfgsfloatval_t fx;
        lbfgs_parameter_t param;

   /* K = kernel( data, ker, arg ) */
   /* ------------------------------------------- */
   if( nrhs == 6)
   {
      if( !mxIsNumeric(prhs[0]) || !mxIsDouble(prhs[0]) ||
        mxIsEmpty(prhs[0])    || mxIsComplex(prhs[0]) )
        mexErrMsgTxt("Matrix Dtrn must be a real matrix.");

      if( !mxIsNumeric(prhs[1]) || !mxIsDouble(prhs[1]) ||
        mxIsEmpty(prhs[1])    || mxIsComplex(prhs[1]) )
        mexErrMsgTxt("Sigma must be a real matrix.");

      if( !mxIsNumeric(prhs[2]) || !mxIsDouble(prhs[2]) ||
        mxIsEmpty(prhs[2])    || mxIsComplex(prhs[2]) )
        mexErrMsgTxt("vector alpha must be a real matrix.");


      /* get pointer at input vectors */

        N = mxGetM(prhs[2]);
        NN = N * N;
        
        {
	const unsigned int nDims = mxGetNumberOfDimensions(prhs[0]);
	const mwSize *dims = mxGetDimensions(prhs[0]);
	if ((N != dims[1]) || (N!=dims[0]))
		mexErrMsgTxt("First argument must be symmetric in first 2 dimensions");
        }
       
	if (mxGetN(prhs[1]) != 1)
		mexErrMsgTxt("Second argument must be of size [1,1]");

	if (mxGetM(prhs[1]) != 1)
		mexErrMsgTxt("Second argument must be of size [1,1]");
	
	if (mxGetM(prhs[2]) != N)
		mexErrMsgTxt("Third argument must be of size [NPoints,1]");

	if (mxGetN(prhs[2]) != 1)
		mexErrMsgTxt("Third argument must be of size [Npoints,1]");



        Ds = mxGetPr(prhs[0]);
        x0 = mxGetPr(prhs[1]);        
        alpha = mxGetPr(prhs[2]);

        if (mxGetM(prhs[3]) != 1)
                mexErrMsgTxt("mxGetM(prhs[3]) != 1");
        if (mxGetN(prhs[3]) != 1)
                mexErrMsgTxt("mxGetN(prhs[3]) != 1");
          maxiters = mxGetPr(prhs[3]);         

        if (mxGetM(prhs[4]) != 1)
                mexErrMsgTxt("mxGetM(prhs[4]) != 1");
        if (mxGetN(prhs[4]) != 1)
                mexErrMsgTxt("mxGetN(prhs[4]) != 1");
        M = (int) (*mxGetPr(prhs[4]));

        if (mxGetM(prhs[5]) != 1)
                mexErrMsgTxt("mxGetM(prhs[5]) != 1");
        if (mxGetN(prhs[5]) != 1)
                mexErrMsgTxt("mxGetN(prhs[5]) != 1");
        Eta = (*mxGetPr(prhs[5]));


                /* creates output vector f. */
                plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
                Sigma   = mxGetPr(plhs[0]);

		/* Initialize the parameters for the L-BFGS optimization. */
		lbfgs_parameter_init(&param);
		param.m = M;
		param.linesearch     = LBFGS_LINESEARCH_BACKTRACKING;
		param.max_linesearch = 20;
		param.max_iterations = *maxiters;
		param.epsilon        = 10e-6;


		lbfgs_x0 = lbfgs_malloc (1);
		lbfgs_x0[0] = sqrt (fabs (x0[0])); //fabs is not necessary

		/*
			Start the L-BFGS optimization; this will invoke the callback functions
			evaluate() and progress() when necessary.
		 */

    	        ret = lbfgs(1, lbfgs_x0, &fx, evaluate, progress, NULL, &param);

		/* Report the result. */
		if (ret!=0 && ret != 1  && ret != 2 && ret != -998)
		        mexPrintf("L-BFGS optimization terminated with status code = %d\n", ret);
		        
		Sigma[0] = lbfgs_x0[0] * lbfgs_x0[0];

		if (nlhs > 1)
		{
			/* creates output vector Sigma. */
			plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
			fval    = mxGetPr(plhs[1]);
			*fval	= fx;
		}

		lbfgs_free(lbfgs_x0);
   }
   else
   {
      mexErrMsgTxt("Wrong number of input arguments.");
   }

   return;
}
