/****************************************************************************
 *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
 *   ghiathi@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mex.h"
#include "matrix.h"
#include <math.h>
#include <stdlib.h>
#include <fstream>
#include "/opt/mosek/5/tools/platform/linux32x86/h/mosek.h" /* Include the MOSEK definition file . */
//#include "C:\Program Files/Mosek/5/tools/platform/win/h/mosek.h" /* Include the MOSEK definition file . */
#include <memory.h>


#define epsm  1e-10
double  *X, *active_features, *x0, *r, *ridx, *Gamma_norm, *Beta, *y, *alpha;
double  teta, *cteta, *steta, *SVM_C, *param_epsilon, *param_past, *param_delta;

long	m, n, N, active_n, active_N;
int*	int_active_features;
int*	int_ridx;

double  *Xptr, *rptr, *ctetaptr, *stetaptr, *xptr, *maxiters, tempfcx, tempfsx;
int		*active_features_ptr, *ridx_ptr;
static int	QPCallCount = 0;
static int     iterations=0;


double  augLagLambda, augLagEtaStar, augLagOmegaStar, augLagEta, augLagMu, augLagOmega;
int     augLagFactr;

void Stabilizer (const double& s, double& value, double& grad)
{
value = exp (-(*Beta) * s * s);
grad  =  -2 * (*Beta) * s * value;
}


static void MSKAPI printstr ( void * handle , char str [])
{
	//mexPrintf("%s\n", str);
}


double MOSEK_QP (double* mu, const double* Gamma)
{
	int		i, j, k;
	MSKenv_t        env ;
	MSKtask_t       task ;
	MSKrescodee     r;
	double 		norm_x, stabilizer, sgrad;
	double		f = 10000;
	double		fDual = 10000;	
	int		sampleNo, cCnstr, cVar;

	cCnstr = 2 * m + 2;
	cVar   = active_N + 2 * m;

	double  *xx    = new double [cVar];
	MSKidxt *qsubi = new MSKidxt [cVar];
	MSKidxt *qsubj = new MSKidxt [cVar];
	double  *qval  = new double [cVar];
	double* c = new double [cVar];
	MSKboundkeye* bkc = new MSKboundkeye [cCnstr];
	double* blc = new double [cCnstr];
	MSKboundkeye* bkx = new MSKboundkeye [cVar];
	double* blx = new double [cVar];
	double* bux = new double [cVar];
	MSKidxt *ptrb = new MSKidxt [cVar];
	MSKidxt *ptre = new MSKidxt [cVar];
	MSKidxt *asub = new MSKidxt [cCnstr * active_N + 2 * m];
	double  *aval = new double [cCnstr * active_N + 2 * m];

	//mexPrintf("Begin Calling SVMQP\n");
	for (i=0; i < active_N; i++)
	{
		qsubi [i] = i; 
		qsubj [i] = i; 
		qval  [i] = 2 * mu[0];
	}

	for (i=0; i < m; i++)
	{
		norm_x = 0;
		for (k = 0; k < active_n; k++)
			norm_x += Gamma[i * active_n + k] * Gamma[i * active_n + k];
		norm_x = sqrt (norm_x);
		Stabilizer (norm_x, stabilizer, sgrad);

		qsubi [active_N + i] = active_N + i; 
		qsubj [active_N + i] = active_N + i; 
		qval  [active_N + i] = 2 * mu[i+1] * stabilizer;
	}

	for (i=0; i < m; i++)
	{
		qsubi [active_N + m + i] = active_N + m + i; 
		qsubj [active_N + m + i] = active_N + m + i; 
		qval  [active_N + m + i] = qval  [active_N + i];
	}

	for (i=0; i<cVar; i++)
		c[i] = 0;
	for (i=0;i<cCnstr;i++)
		bkc[i] = MSK_BK_FX;
	for (i=0; i<cCnstr; i++)
		blc[i] = 0;

	blc[0] = 2;
	// buc = blc

	for (i=0; i<active_N; i++)
	{
		bkx[i] = MSK_BK_RA;
		blx[i] = 0;
		bux[i]= *SVM_C;
	}
	for (i=active_N; i<cVar; i++)
	{
		bkx[i] = MSK_BK_FR;
		blx[i] = -MSK_INFINITY;
		bux[i] = MSK_INFINITY;
	}

	for (i=0; i < active_N;i++)
	{
		sampleNo = int_ridx[i]-1;
		ptrb[i] = cCnstr * i;
		ptre[i] = cCnstr * (i + 1);		
		asub[cCnstr * i] = 0;
		aval[cCnstr * i] = 1;
		asub[cCnstr * i + 1] = 1;
		aval[cCnstr * i + 1] = y[sampleNo];
		for (j=0; j < m;j++)
		{
			double teta = 0;
			for (k=0; k < active_n; k++)		
				teta += X[sampleNo * n + k] * Gamma[j * active_n + k];

			asub[cCnstr * i + 2 + j] = 2 + j;
			aval[cCnstr * i + 2 + j] = y[sampleNo] * cos(teta);
			asub[cCnstr * i + 2 + m + j] = 2 + m + j;
			aval[cCnstr * i + 2 + m + j] = y[sampleNo] * sin(teta);
		}
	}

	for (i=0; i < m;i++)
	{
		ptrb[active_N + i] = cCnstr * active_N + i;
		ptre[active_N + i] = cCnstr * active_N + i + 1;	
		asub[cCnstr * active_N + i] = 2 + i;
		aval[cCnstr * active_N + i] = -1;
	}

	for (i=0; i < m;i++)
	{
		ptrb[active_N+m+i] = cCnstr * active_N + m + i;
		ptre[active_N+m+i] = cCnstr * active_N + m + i + 1;	
		asub[cCnstr * active_N + m + i] = 2 + m + i;
		aval[cCnstr * active_N + m + i] = -1;
	}


	// Create the mosek environment . 
	r = MSK_makeenv (& env , NULL , NULL , NULL , NULL );
	// Check whether the return code is ok . 
	if ( r == MSK_RES_OK )
	{
	// Directs the log stream to the ’ printstr ’ function . 
	MSK_linkfunctoenvstream ( env ,
					MSK_STREAM_LOG ,
					NULL ,
					printstr );
	}
	// Initialize the environment .
	r = MSK_initenv ( env );
	if ( r == MSK_RES_OK )
	{
	// Create the optimization task . 
	r = MSK_maketask ( env , cCnstr , cVar ,&task );
	if ( r == MSK_RES_OK )
	{
	r = MSK_linkfunctotaskstream ( task , MSK_STREAM_LOG , NULL , printstr );
	if ( r == MSK_RES_OK )
	{
		r = MSK_inputdata ( task ,
			cCnstr , cVar ,
			cCnstr , cVar ,
			c ,0.0 ,
			ptrb ,
			ptre ,
			asub ,
			aval ,
			bkc ,
			blc ,
			blc ,	//buc = blc
			bkx ,
			blx ,
			bux );
	}
	if ( r == MSK_RES_OK )
	{
		// Input the Q for the objective .
		r = MSK_putqobj ( task , cVar , qsubi , qsubj , qval );
	}
	if ( r == MSK_RES_OK )
	{
		r = MSK_putdouparam ( task , MSK_DPAR_INTPNT_TOL_REL_GAP , 1.0e-14 );
	}
	if ( r == MSK_RES_OK )
		r = MSK_optimize ( task );
	if ( r == MSK_RES_OK )
	{
		MSK_getsolutionslice ( task ,
				MSK_SOL_ITR ,
				MSK_SOL_ITEM_XX ,
				0,
				cVar ,
				xx );

		MSK_getprimalobj (task, MSK_SOL_ITR, &f );
		//MSK_getdualobj (task, MSK_SOL_ITR, &fDual );		
		//mexPrintf ("Primal - Dual = %10.10f\n", fDual - f);
		
		for (i=0;i < active_N;i++)
		{
			sampleNo = int_ridx[i]-1;
			alpha[i] = xx[i];
			::r[i] = xx[i] * y[sampleNo];
		}

		//for ( j =0; j < cVar ; ++ j )
		//    mexPrintf ("x [%d ]: %e \n " ,j , xx[j]);
	}
	}
	MSK_deletetask (& task );
	}
	MSK_deleteenv (& env );

	delete[] xx;
	delete[] qsubi;
	delete[] qsubj;
	delete[] qval;
	delete[] c;
	delete[] bkc;
	delete[] blc;
	delete[] bkx;
	delete[] blx;
	delete[] bux;
	delete[] ptrb;
	delete[] ptre;
	delete[] asub;
	delete[] aval;

	if (r)
		mexPrintf (" vaaai:  Return code : % d \n " , r );

	QPCallCount++;
	//mexPrintf ("QP Call Count: %d \n " , QPCallCount);
		// mexPrintf("End Calling SVMQP\n");
	return f;
}


static double evaluateAlphaGAlpha(const double *gamma,  double *gammaGrad)
{
	int i, j, sampleNo;
	double f;
	double norm_x = 0;
	double stabilizer;
	double sgrad;
	double fc;
	double fs;
	double temp;
	double epsilon = 0.0001;
	double sq;
	double *x = (double*)((void*)gamma), *g = gammaGrad;

	if (*Gamma_norm == 1)
	{
		for (i = 0; i < active_n; i++)
		{
			if (fabs(x[i]) > epsilon)
				norm_x += fabs(x[i]) - epsilon / 2;
			else
				norm_x += x[i] * x[i] / (2 * epsilon);
		}
	} else if (*Gamma_norm == 2)
	{
		for (i = 0; i < active_n; i++)
			norm_x += x[i] * x[i];
		norm_x = sqrt (norm_x);
	}

	Stabilizer (norm_x, stabilizer, sgrad);

	fc = 0;
	fs = 0;
	ridx_ptr = &int_ridx[0];
	ctetaptr = &cteta[0];
	stetaptr = &steta[0];
	rptr = &r[0];
	for (i = 0; i < active_N ; i++, ctetaptr++, stetaptr++, ridx_ptr++, rptr++)
	{
		teta = 0;
		sampleNo = *ridx_ptr - 1;
		Xptr = &X[sampleNo * n - 1];
		active_features_ptr = &int_active_features[0];
		for (j=0; j<active_n; j++, active_features_ptr++)
			teta += Xptr[*active_features_ptr] * x[j];

		*ctetaptr = cos (teta);
		*stetaptr = sin (teta);

		fc += *rptr * (*ctetaptr);
		fs += *rptr * (*stetaptr);
	}

	sq = (fc * fc + fs * fs);
	f = - sq * stabilizer;

	if (norm_x < 10e-6)
		norm_x = 1; // to avoid NaN

	for (j = 0; j < active_n; j++)
	{
	tempfcx = 0;
	tempfsx = 0;
	rptr = &r[0];
	ridx_ptr = &int_ridx[0];
	ctetaptr = &cteta[0];
	stetaptr = &steta[0];
	for (i = 0; i < active_N ; i++, ctetaptr++, stetaptr++, ridx_ptr++, rptr++)
	{
		sampleNo = *ridx_ptr - 1;
		Xptr = &X[sampleNo * n + int_active_features[j] - 1];
		temp    = *Xptr * (*rptr);
		tempfcx += temp * (*ctetaptr);
		tempfsx += temp * (*stetaptr);
	}

	g[j] = 2 * (fc * tempfsx - fs * tempfcx) * stabilizer;

	if (*Gamma_norm == 2)
        	g[j] -= sq * sgrad * x[j] / norm_x;
	else if (*Gamma_norm == 1)
	{
		if (fabs(x[j]) > epsilon)
			g[j] += sgrad * f * (x[j] > 0 ? 1 : -1);
		else
		{
		/*
                	mxArray *rhs[1];
                	double *rhsC, *lhsC1, *lhsC2;		
		        mexPrintf ("What hapened?");
                        rhs[0]    = mxCreateDoubleMatrix(1, 1, mxREAL);
                        rhsC   = mxGetPr (rhs[0]);
		        rhsC[0] = 0;
		        mexCallMATLAB(0, NULL, 1, rhs, "userNotification");
		        mxDestroyArray(rhs[0]);			
                */
			g[j] += sgrad * f * x[j] / epsilon;
		}
	}

	}

return f;
}

// Compute the gradient of -J with respect o mu (not rho)
static double ComputeMinusJAndGradientMosek(int cVar, const double *x, double *g)
{
	int i, j;
	double f,ftemp;
	double norm_x = 0;
	double* mu = (double*)((void*)x);

	f = MOSEK_QP (mu, mu+m+1);
        
	double aTa = 0;
	for (i=0;i < active_N; i++)
		aTa += r[i] * r[i];
		
        g[0] = aTa;
        for (i = 1; i <= m; i++)
        {
	        ftemp = evaluateAlphaGAlpha (&x[m + 1 + (i-1) * active_n],&g[m + 1 + (i-1) * active_n]);
	        g[i] = -ftemp;
	        for (j=0;j<active_n; j++)
	                g[m + 1 + (i-1)*active_n +j] *=  mu[i];
        }

        return -f;
}


static double evaluate(void *instance, const double *mu,  double *g, const int param_n, const double step)
{
	int    i, j;
	double MinusJ, sumRho2;
	double *gJ;

	gJ     = new double[param_n];
	MinusJ = ComputeMinusJAndGradientMosek(param_n, mu, gJ);
	memcpy (g, gJ, param_n * sizeof(double));
	
	delete gJ;
	return MinusJ;
}


/* ==============================================================
Main MEX function - interface to Matlab.
============================================================== */
void mexFunction( int nlhs, mxArray *plhs[],
		int nrhs, const mxArray *prhs[] )
{
	long	i, j, ret;
	int		M;
	int		maxits;
	double *outval;


/* K = kernel( data, ker, arg ) */
/* ------------------------------------------- */
if( nrhs == 9)
{
/* data matrix [n x N] */
if( !mxIsNumeric(prhs[0]) || !mxIsDouble(prhs[0]) ||
	mxIsEmpty(prhs[0])      || mxIsComplex(prhs[0]) )
	mexErrMsgTxt("Input data must be a real matrix.");

/* active_features [1 x active_n] */
if( !mxIsNumeric(prhs[1]) || !mxIsDouble(prhs[1]) ||
	mxIsEmpty(prhs[1])    || mxIsComplex(prhs[1]) )
	mexErrMsgTxt("active_features must be a real matrix.");

/* data matrix [1 x n] */
if( !mxIsNumeric(prhs[2]) || !mxIsDouble(prhs[2]) ||
	mxIsEmpty(prhs[2])    || mxIsComplex(prhs[2]) )
	mexErrMsgTxt("vector x0 must be a real matrix.");


/* get pointer at input vectors */
X = mxGetPr(prhs[0]);
n = mxGetM (prhs[0]);
N = mxGetN (prhs[0]);

active_features = mxGetPr(prhs[1]);
if (mxGetM(prhs[1]) != 1)
	mexErrMsgTxt("size(active_features,1) must be 1.");
active_n = mxGetN(prhs[1]);
int_active_features = new int [active_n];
for (j = 0; j < active_n; j++)
	int_active_features[j] = (int) active_features[j];

x0 = mxGetPr(prhs[2]);
if ((mxGetM(prhs[2])-1) % (active_n+1) != 0)
	mexErrMsgTxt("size(x0,1) must be m*(active_n+1)+1.");
m = (mxGetM(prhs[2])-1) / (active_n + 1);
if (mxGetN(prhs[2]) != 1)
	mexErrMsgTxt("size(x0,2) must be 1.");

active_N = N;
alpha = new double [N];
r	    = new double [N];
int_ridx = new int [N];
for (j = 0; j < active_N; j++)
	int_ridx[j] = j+1;

Gamma_norm = mxGetPr(prhs[3]);
if (mxGetM(prhs[3]) != 1)
	mexErrMsgTxt("mxGetM(prhs[3]) != 1");
if (mxGetN(prhs[3]) != 1)
	mexErrMsgTxt("mxGetN(prhs[3]) != 1");

Beta = mxGetPr(prhs[4]);
if (mxGetM(prhs[4]) != 1)
	mexErrMsgTxt("mxGetM(prhs[4]) != 1");
if (mxGetN(prhs[4]) != 1)
	mexErrMsgTxt("mxGetN(prhs[4]) != 1");

	cteta = mxGetPr(prhs[5]);
if (mxGetM(prhs[5]) != 1)
	mexErrMsgTxt("mxGetM(prhs[5]) != 1");
if (mxGetN(prhs[5]) != N)
	mexErrMsgTxt("mxGetN(prhs[5]) != N");

	steta = mxGetPr(prhs[6]);
if (mxGetM(prhs[6]) != 1)
	mexErrMsgTxt("mxGetM(prhs[6]) != 1");
if (mxGetN(prhs[6]) != N)
	mexErrMsgTxt("mxGetN(prhs[6]) != N");

	y = mxGetPr(prhs[7]);
if (mxGetM(prhs[7]) != 1)
	mexErrMsgTxt("mxGetM(prhs[7]) != 1");
if (mxGetN(prhs[7]) != N)
	mexErrMsgTxt("mxGetN(prhs[7]) != N");

	SVM_C = mxGetPr(prhs[8]);
if (mxGetM(prhs[8]) != 1)
	mexErrMsgTxt("mxGetM(prhs[8]) != 1");
if (mxGetN(prhs[8]) != 1)
	mexErrMsgTxt("mxGetN(prhs[8]) != 1");



		double *gRet = new double [m*(active_n+1)+1];
		double fRet = evaluate(NULL, x0,  gRet, m*(active_n+1)+1, 0);
		
		if (nlhs > 0)
		{
			plhs[0]   = mxCreateDoubleMatrix(1,1,mxREAL);
			outval    = mxGetPr(plhs[0]);
			*outval	  = fRet;

		        if (nlhs > 1)
		        {
			        plhs[1] = mxCreateDoubleMatrix(1,m*(active_n+1)+1,mxREAL);
			        outval    = mxGetPr(plhs[1]);
			        for (i=0; i < m*(active_n+1)+1; i++)
				        outval[i] = gRet[i];
		        }						
		}

		delete[] gRet;		
		delete[] int_active_features;
		delete[] alpha;
		delete[] r;
		delete[] int_ridx;
}
else
{
        mexErrMsgTxt("Wrong number of input arguments.");
}

return;
}
