/****************************************************************************
 *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
 *   ghiathi@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
# include <stdio.h>
# include " mosek.h " /* Include the MOSEK definition file . */

static void MSKAPI printstr ( void * handle , char str [])
{
   printf ("% s " , str );
} /* printstr */


double SVMQP (const int m, const double* rho, const double* Gamma, double* optimalAlpha)
{
	MSKidxt                   j;
	double                    *xx;
	MSKenv_t                  env ;
	MSKtask_t                 task ;
	MSKrescodee               r;

	cCnstr = 2 * m + 2;
	cVar   = n + 2 * m;

	xx = new double[cVar];
	qsubi = new double [cVar];
	qsubj = new double [cVar];
	qval  = new double [cVar];

	for (i=0; i < n; i++)
	{
	qsubi [i] = i; 
	qsubj [i] = i; 
	qval [i] = 2 * model.options.tau;
	}

	for (i=0; i < m; i++)
	{
	qsubi [n + i] = n + i; 
	qsubj [n + i] = n + i; 
	qval [n + i] = 2 * model.options.arg2(i);
	}

	for (i=0; i < m; i++)
	{
	qsubi [n + m + i] = n + m + i; 
	qsubj [n + m + i] = n + m + i; 
	qval  [n + m + i] = 2 * model.options.arg2(i);
	}

	double* c = new double [cVar];
	memset (c,0,cVar);

	MSKboundkeye* bkc = new MSKboundkeye [cCnstr];
	double* blc = new double [cCnstr];
	memset (blc,0,cCnstr);
	blc[0] = 2;
	// buc = blc
	double* blx = new double [cVar];
	double* bux = new double [cVar];

	for (i=0; i<n; i++)
	{
		bkc[i] = MSK_BK_LO;
		blx[i] = 0
		bux[i]= MSK_INFINITY;
	}
	for (i=n; i<cVar; i++)
	{
		bkc[i] = MSK BK FR;
		blx[i] = -MSK_INFINITY;
		bux[i] = MSK_INFINITY;
	}

	MSKidxt *ptrb=new double [cVar];
	MSKidxt *ptre=new double [cVar];
	MSKidxt *asub=new double [2 * n];
	double *aval=new double [2 * n];

	for (i=0; i < n;i++)
	{
		ptrb[i] = cCnstr * i;
		ptre[i] = cCnstr * (i + 1);		
		asub[cCnstr * i] = 0;
		aval[cCnstr * i] = 1;
		asub[cCnstr * i + 1] = 1;
		aval[cCnstr * i + 1] = y(i);
		for (j=0; j < m;j++)
		{
			double teta = 0;
			for (k=0; j < n;k++)		
				teta += trn.X[i,k] * Gamma[j,k];
			asub[cCnstr * i + 2 + j] = 2 + j;
			aval[cCnstr * i + 2 + j] = y(i) * cos(teta);
			asub[cCnstr * i + 2 + m + j] = 2 + m + j;
			aval[cCnstr * i + 2 + m + j] = y(i) * sin(teta);
		}
	}

	for (i=0; i < m;i++)
	{
		ptrb[n+i] = cCnstr * n + i;
		ptre[n+i] = cCnstr * n + i + 1;	
		asub[2 * i] = 3 + i;
		aval[2 * i] = -1;
	}

	for (i=0; i < m;i++)
	{
		ptrb[n+m+i] = cCnstr * n + m + i;
		ptre[n+m+i] = cCnstr * n + m + i + 1;	
		asub[2 * i] = 3 + m + i;
		aval[2 * i] = -1;
	}



	/* Create the mosek environment . */
	r = MSK_makeenv (& env , NULL , NULL , NULL , NULL );
	/* Check whether the return code is ok . */
	if ( r == MSK_RES_OK )
	{
	  /* Directs the log stream to the ’ printstr ’ function . */
	  MSK_linkfunctoenvstream ( env ,
                                  MSK_STREAM_LOG ,
                                  NULL ,
                                  printstr );
	}
	/* Initialize the environment . */
	r = MSK_initenv ( env );
	if ( r == MSK_RES_OK )
	{
	/* Create the optimization task . */
	  r = MSK_maketask ( env , NUMCON , NUMVAR ,& task );
	  if ( r == MSK_RES_OK )
	  {
	      r = MSK_linkfunctotaskstream ( task , MSK_STREAM_LOG , NULL , printstr );
	      if ( r == MSK_RES_OK )
	      {
		  r = MSK_inputdata ( task ,
                      NUMCON , NUMVAR ,
                      NUMCON , NUMVAR ,
                      c ,0.0 ,
                      ptrb ,
                      ptre ,
                      asub ,
                      aval ,
                      bkc ,
                      blc ,
                      buc ,
                      bkx ,
                      blx ,
                      bux );
	      }
	      if ( r == MSK_RES_OK )
	      {
		 /* Input the Q for the objective . */
		 r = MSK_putqobj ( task , cVar , qsubi , qsubj , qval );
	      }
	      if ( r == MSK_RES_OK )
		 r = MSK_optimize ( task );
	      if ( r == MSK_RES_OK )
	      {
		 MSK_getsolutionslice ( task ,
	                           MSK_SOL_ITR ,
	                           MSK_SOL_ITEM_XX ,
	                           0,
	                           NUMVAR ,
	                           xx );

		for (i=0;i < n;i++)
			optimalAlpha[i] = xx[i];
		 printf (" Primal solution \ n ");
		 for ( j =0; j < NUMVAR ; ++ j )
		     printf (" x [% d ]: % e \ n " ,j , xx [ j ]);
	      }
	    }
	    MSK_deletetask (& task );
	  }
	  MSK_deleteenv (& env );

	  delete[] xx;
	  delete[] qsubi;
	  delete[] qsubj;
	  delete[] qval;
	  delete[] c;
	  delete[] bkc;
	  delete[] blc;
	  delete[] blx;
	  delete[] bux;
	  delete[] ptrb;
	  delete[] ptre;
	  delete[] asub;
	  delete[] aval;

	  printf (" Return code : % d \ n " , r );
	  return ( r );
} 


