/****************************************************************************
 *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
 *   ghiathi@gmail.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mex.h"
#include "matrix.h"
#include <math.h>
#include <stdlib.h>
#include <fstream>
#include "lbfgs.h"

double *X, *active_features, *x0, *r, *ridx, *Gamma_norm, *Beta;
double teta, *cteta, *steta;

long	n, N, active_n, active_N;
int*	int_active_features;
int*	int_ridx;

double  *Xptr, *rptr, *ctetaptr, *stetaptr, *xptr, *maxiters, tempfcx, tempfsx;
int		*active_features_ptr, *ridx_ptr;

double  *param_epsilon, *param_past, *param_delta;

// Although in the paper, Stabilizer is multiplicative, here, Stabilizer is in denumerator.
void Stabilizer (const double& s, double& value, double& grad)
{
    value = exp ((*Beta) * s * s);
    grad  =  2 * (*Beta) * s * value;
}


static int progress(
    void *instance,
    const lbfgsfloatval_t *x,
    const lbfgsfloatval_t *g,
    const lbfgsfloatval_t fx,
    const lbfgsfloatval_t xnorm,
    const lbfgsfloatval_t gnorm,
    const lbfgsfloatval_t step,
    int n,
    int k,
    int ls
    )
{
    //mexPrintf("Iteration: %d\t", k);
    //mexPrintf("  fx = %f, x[0] = %f, x[1] = %f\t", fx, x[0], x[1]);
    //mexPrintf("  xnorm = %f, gnorm = %f, step = %f\n", xnorm, gnorm, step);
    return 0;
}


static lbfgsfloatval_t evaluate(void *instance, const lbfgsfloatval_t *x,  lbfgsfloatval_t *g,
    const int n, const lbfgsfloatval_t step /*, const lbfgsfloatval_t *xp */)
{
	int i, j, sampleNo;
	lbfgsfloatval_t f;
	lbfgsfloatval_t norm_x = 0;
	lbfgsfloatval_t stabilizer;
	lbfgsfloatval_t sgrad;
	lbfgsfloatval_t fc;
	lbfgsfloatval_t fs;
	lbfgsfloatval_t temp;
	lbfgsfloatval_t epsilon = 0.0001;

	if (*Gamma_norm == 1)
	{
		for (i = 0; i < active_n; i++)
		{
			if (fabs(x[i]) > epsilon)
				norm_x += fabs(x[i]) - epsilon / 2;
			else
				norm_x += x[i] * x[i] / (2 * epsilon);
		}
	}
	else if (*Gamma_norm == 2)
	{
		for (i = 0; i < active_n; i++)
			norm_x += x[i] * x[i];
		norm_x = sqrt (norm_x);
	}

    Stabilizer (norm_x, stabilizer, sgrad);

	fc = 0;
	fs = 0;
	ridx_ptr = &int_ridx[0];
	ctetaptr = &cteta[0];
	stetaptr = &steta[0];
	rptr = &r[0];
	for (i = 0; i < active_N ; i++, ctetaptr++, stetaptr++, ridx_ptr++, rptr++)
	{
		teta = 0;
		sampleNo = *ridx_ptr - 1;
		Xptr = &X[sampleNo * n - 1];
		active_features_ptr = &int_active_features[0];
		for (j=0; j<active_n; j++, active_features_ptr++)
			teta += Xptr[*active_features_ptr] * x[j];

		*ctetaptr = cos (teta);
		*stetaptr = sin (teta);

		fc += *rptr * (*ctetaptr);
		fs += *rptr * (*stetaptr);
	}
	f = - (fc * fc + fs * fs) / stabilizer;

	if (norm_x < 10e-6)
		norm_x = 1; // to avoid NaN

	for (j = 0; j < active_n; j++)
	{
	  tempfcx = 0;
	  tempfsx = 0;
	  rptr = &r[0];
	  ridx_ptr = &int_ridx[0];
	  ctetaptr = &cteta[0];
	  stetaptr = &steta[0];
	  for (i = 0; i < active_N ; i++, ctetaptr++, stetaptr++, ridx_ptr++, rptr++)
	  {
		sampleNo = *ridx_ptr - 1;
		Xptr = &X[sampleNo * n + int_active_features[j] - 1];
		temp    = *Xptr * (*rptr);
		tempfcx += temp * (*ctetaptr);
		tempfsx += temp * (*stetaptr);
	  }

	g[j] = (2 * fc * tempfsx - 2 * fs * tempfcx) / stabilizer;

	if (*Gamma_norm == 2)
	    g[j] -= sgrad * f / stabilizer * x[j] / norm_x;
	else if (*Gamma_norm == 1)
	{
		if (fabs(x[j]) > epsilon)
			g[j] -= sgrad * f / stabilizer * (x[j] > 0 ? 1 : -1);
		else
			g[j] -= sgrad * f / stabilizer * x[j] / epsilon;
	}
	}

return f;
}


/* ==============================================================
 Main MEX function - interface to Matlab.
============================================================== */
void mexFunction( int nlhs, mxArray *plhs[],
		  int nrhs, const mxArray *prhs[] )
{
	long	i, j, ret;
	int		M;
	double  epsg = 1e-6;
	double  epsf = 1e-6;
	double  epsx = 1e-6;
	int		maxits;
	int		info;
	double *Gamma, *fval;
    lbfgsfloatval_t fx;
    lbfgs_parameter_t param;

   /* K = kernel( data, ker, arg ) */
   /* ------------------------------------------- */
   if( nrhs == 14)
   {
      /* data matrix [n x N] */
      if( !mxIsNumeric(prhs[0]) || !mxIsDouble(prhs[0]) ||
        mxIsEmpty(prhs[0])    || mxIsComplex(prhs[0]) )
        mexErrMsgTxt("Input data must be a real matrix.");

      /* active_features [1 x active_n] */
      if( !mxIsNumeric(prhs[1]) || !mxIsDouble(prhs[1]) ||
        mxIsEmpty(prhs[1])    || mxIsComplex(prhs[1]) )
        mexErrMsgTxt("active_features must be a real matrix.");

      /* data matrix [1 x n] */
      if( !mxIsNumeric(prhs[2]) || !mxIsDouble(prhs[2]) ||
        mxIsEmpty(prhs[2])    || mxIsComplex(prhs[2]) )
        mexErrMsgTxt("vector x0 must be a real matrix.");


      /* get pointer at input vectors */
      X = mxGetPr(prhs[0]);
      n = mxGetM (prhs[0]);
      N = mxGetN (prhs[0]);

      active_features = mxGetPr(prhs[1]);
      if (mxGetM(prhs[1]) != 1)
         mexErrMsgTxt("size(active_features,1) must be 1.");
      active_n = mxGetN(prhs[1]);
      int_active_features = new int [active_n];
      for (j = 0; j < active_n; j++)
      	int_active_features[j] = (int) active_features[j];

      x0 = mxGetPr(prhs[2]);
      if (mxGetM(prhs[2]) != active_n)
         mexErrMsgTxt("size(x0,1) must be active_n.");
      if (mxGetN(prhs[2]) != 1)
         mexErrMsgTxt("size(x0,2) must be 1.");

      r = mxGetPr(prhs[3]);
      if (mxGetM(prhs[3]) != 1)
         mexErrMsgTxt("mxGetM(prhs[3]) != 1");
      active_N = mxGetN(prhs[3]);

      ridx = mxGetPr(prhs[4]);
      if (mxGetM(prhs[4]) != 1)
         mexErrMsgTxt("mxGetM(prhs[4]) != 1");
      if (mxGetN(prhs[4]) != active_N)
         mexErrMsgTxt("mxGetN(prhs[4]) != active_N");
      int_ridx = new int [active_N];
      for (j = 0; j < active_N; j++)
      	int_ridx[j] = (int) ridx[j];

      Gamma_norm = mxGetPr(prhs[5]);
      if (mxGetM(prhs[5]) != 1)
         mexErrMsgTxt("mxGetM(prhs[5]) != 1");
      if (mxGetN(prhs[5]) != 1)
         mexErrMsgTxt("mxGetN(prhs[5]) != 1");

      Beta = mxGetPr(prhs[6]);
      if (mxGetM(prhs[6]) != 1)
         mexErrMsgTxt("mxGetM(prhs[6]) != 1");
      if (mxGetN(prhs[6]) != 1)
         mexErrMsgTxt("mxGetN(prhs[6]) != 1");

	  maxiters = mxGetPr(prhs[7]);
      if (mxGetM(prhs[7]) != 1)
         mexErrMsgTxt("mxGetM(prhs[7]) != 1");
      if (mxGetN(prhs[7]) != 1)
         mexErrMsgTxt("mxGetN(prhs[7]) != 1");
	  maxits = (int) (*maxiters);

	  M = (int) (*mxGetPr(prhs[8]));
	  if (active_n < M)
	  	M = active_n;
      if (mxGetM(prhs[8]) != 1)
         mexErrMsgTxt("mxGetM(prhs[8]) != 1");
      if (mxGetN(prhs[8]) != 1)
         mexErrMsgTxt("mxGetN(prhs[8]) != 1");


	  cteta = mxGetPr(prhs[9]);
      if (mxGetM(prhs[9]) != 1)
         mexErrMsgTxt("mxGetM(prhs[9]) != 1");
      if (mxGetN(prhs[9]) != N)
         mexErrMsgTxt("mxGetN(prhs[9]) != N");

	  steta = mxGetPr(prhs[10]);
      if (mxGetM(prhs[10]) != 1)
         mexErrMsgTxt("mxGetM(prhs[10]) != 1");
      if (mxGetN(prhs[10]) != N)
         mexErrMsgTxt("mxGetN(prhs[10]) != N");

	  steta = mxGetPr(prhs[10]);
      if (mxGetM(prhs[10]) != 1)
         mexErrMsgTxt("mxGetM(prhs[10]) != 1");
      if (mxGetN(prhs[10]) != N)
         mexErrMsgTxt("mxGetN(prhs[10]) != N");

	param_epsilon = mxGetPr(prhs[11]);
if (mxGetM(prhs[11]) != 1)
	mexErrMsgTxt("mxGetM(prhs[11]) != 1");
if (mxGetN(prhs[11]) != 1)
	mexErrMsgTxt("mxGetN(prhs[11]) != 1");

	param_past = mxGetPr(prhs[12]);
if (mxGetM(prhs[12]) != 1)
	mexErrMsgTxt("mxGetM(prhs[12]) != 1");
if (mxGetN(prhs[12]) != 1)
	mexErrMsgTxt("mxGetN(prhs[12]) != 1");

	param_delta = mxGetPr(prhs[13]);
if (mxGetM(prhs[13]) != 1)
	mexErrMsgTxt("mxGetM(prhs[13]) != 1");
if (mxGetN(prhs[13]) != 1)
	mexErrMsgTxt("mxGetN(prhs[13]) != 1");


	/*
	  {
		  using namespace std;
		fstream fvariables, fX, fint_active_features, fx0, fr, fint_ridx;

		fvariables.open ("Variables.txt", ios::out| ios::binary);
		fX.open ("X.bin", ios::out| ios::binary);
		fint_active_features.open ("int_active_features.bin", ios::out| ios::binary);
		fx0.open ("x0.bin", ios::out| ios::binary);
		fr.open ("r.bin", ios::out| ios::binary);
		fint_ridx.open ("int_ridx.bin", ios::out| ios::binary);

		fvariables << "n" << "\t" << n << "\n";
		fvariables << "N" << "\t" << N << "\n";
		fvariables << "active_n" << "\t" << active_n << "\n";
		fvariables << "active_N" << "\t" << active_N << "\n";
		fvariables << "Gamma_norm" << "\t" << *Gamma_norm << "\n";
		fvariables << "Beta" << "\t" << *Beta << "\n";
		fvariables << "Maxits" << "\t" << maxits << "\n";


		fX.write ((char*)X, n * N * sizeof (double));
		fint_active_features.write ((char*)int_active_features, active_n * sizeof (int));
		fx0.write ((char*)x0, n * sizeof (double));
		fr.write ((char*)r, active_N * sizeof (double));
		fint_ridx.write ((char*)int_ridx, active_N * sizeof (int));

		fvariables.close();
		fX.close();
		fint_active_features.close();
		fx0.close();
		fr.close();
		fint_ridx.close();

	  }
	  */

        /* creates output vector f. */
	    plhs[0] = mxCreateDoubleMatrix(active_n, 1, mxREAL);
	    Gamma   = mxGetPr(plhs[0]);

	/* Initialize the parameters for the L-BFGS optimization. */
	lbfgs_parameter_init(&param);
	param.m = M;
	param.linesearch = LBFGS_LINESEARCH_BACKTRACKING;
	param.max_linesearch = 20;
        param.epsilon	     = *param_epsilon;
        param.past	     = *param_past;
        param.delta	     = *param_delta;

	/* param.linesearch = LBFGS_LINESEARCH_BACKTRACKING; */

	/*
		Start the L-BFGS optimization; this will invoke the callback functions
		evaluate() and progress() when necessary.
	 */

	lbfgsfloatval_t *lbfgs_x0 = lbfgs_malloc (active_n);
	for (i=0; i < active_n; i++)
		lbfgs_x0[i] = x0[i];
		
    	ret = lbfgs(active_n, lbfgs_x0, &fx, evaluate, progress, NULL, &param);

		/* Report the result. */
		if (ret!=0 && ret != 1  && ret != 2 && ret != -999)
			mexPrintf("L-BFGS optimization terminated with status code = %d\n", ret);

		for (i=0; i < active_n; i++)
			Gamma[i] = lbfgs_x0[i];

		if (nlhs > 1)
		{
			/* creates output vector Gamma. */
			plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
			fval    = mxGetPr(plhs[1]);
			*fval	= fx;
		}

		lbfgs_free(lbfgs_x0);
		delete[] int_active_features;
		delete[] int_ridx;
   }
   else
   {
      mexErrMsgTxt("Wrong number of input arguments.");
   }

   return;
}
