%/***************************************************************************
% *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
% *   ghiathi@gmail.com                                                     *
% *                                                                         *
% *   This program is free software; you can redistribute it and/or modify  *
% *   it under the terms of the GNU General Public License as published by  *
% *   the Free Software Foundation; either version 3 of the License, or     *
% *   (at your option) any later version.                                   *
% *                                                                         *
% *   This program is distributed in the hope that it will be useful,       *
% *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
% *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
% *   GNU General Public License for more details.                          *
% *                                                                         *
% *   You should have received a copy of the GNU General Public License     *
% *   along with this program; if not, write to the                         *
% *   Free Software Foundation, Inc.,                                       *
% *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
% ***************************************************************************/

#include "mex.h"
#include "matrix.h"
#include <math.h>
#include <stdlib.h>
#include "kernel_fun.h"

/* ==============================================================
 Main MEX function - interface to Matlab.
============================================================== */
void mexFunction( int nlhs, mxArray *plhs[],
		  int nrhs, const mxArray *prhs[] )
{
   long i, j, k, l;
   long m, n, new_dim, num_data;
   double x;
   double *Mus;
   double *Gammas;
   double *CosineCoef;
   double *SineCoef;
   double *b;
   double *Y;
   double k_ij;


   ker_cnt = 0;

   /* Y = cosine_kernelproj_mex(X, b, ker, CosineCoef, SineCoef, Gammas, Mus ) */
   /* ------------------------------------------- */
   if( nrhs == 7)
   {
      /* data matrix [dim x num_data] */
      if( !mxIsNumeric(prhs[0]) || !mxIsDouble(prhs[0]) ||
        mxIsEmpty(prhs[0])    || mxIsComplex(prhs[0]) )
        mexErrMsgTxt("Input data must be a real matrix.");

      /* vector b [nsv  x 1] */
      if( !mxIsNumeric(prhs[1]) || !mxIsDouble(prhs[1]) ||
        mxIsEmpty(prhs[1])    || mxIsComplex(prhs[1]) )
        mexErrMsgTxt("Input b must be a real vector.");

      /* kernel identifier */
      ker = kernel_id( prhs[2] );
      if( ker == -1 )
        mexErrMsgTxt("Improper kernel identifier.");

      /* multipliers CosineCoef [1 x m] */
      if( !mxIsNumeric(prhs[3]) || !mxIsDouble(prhs[3]) ||
        mxIsEmpty(prhs[3])    || mxIsComplex(prhs[3]) )
        mexErrMsgTxt("Input Alpha must be a real matrix.");

      /* multipliers SineCoef [1 x m] */
      if( !mxIsNumeric(prhs[4]) || !mxIsDouble(prhs[4]) ||
        mxIsEmpty(prhs[4])    || mxIsComplex(prhs[4]) )
        mexErrMsgTxt("Input Alpha must be a real matrix.");


     /* get data dimensions */
     n = mxGetM(prhs[0]);
     num_data = mxGetN(prhs[0]);
     new_dim = mxGetN(prhs[1]);

     /* get pointer at input vectors */
     dataA = mxGetPr(prhs[0]);
     b 	   = mxGetPr(prhs[1]);
     CosineCoef = mxGetPr(prhs[3]);
	 SineCoef = mxGetPr(prhs[4]);


     /*  get pointer to arguments  */
     Gammas = mxGetPr(prhs[5]);

     if( mxGetM(prhs[5]) != n)
        mexErrMsgTxt("Number of rows of Gammas must be equal to n");
     m = mxGetN(prhs[5]); /* must be m */

     /*  get pointer to arguments  */
     Mus = mxGetPr(prhs[6]);
     if( mxGetM(prhs[6]) != 1)
        mexErrMsgTxt("Number of rows of Mus must be equal to 1");
     if( mxGetN(prhs[6]) != m)
        mexErrMsgTxt("Number of columns of Mus must be equal to m");

     /* creates output kernel matrix. */
     plhs[0] = mxCreateDoubleMatrix(new_dim,num_data,mxREAL);
     Y = mxGetPr(plhs[0]);

     /* computes kernel projection */
     for( i = 0; i < num_data; i++ ) {

       for( k = 0; k < new_dim; k++) {
         Y[k+i*new_dim] = b[k];
	     for( j = 0; j < m; j++ ) {
			 x = 0;
			 for( l = 0; l < n; l++ ) {
				 x += Gammas[l + j * n] * dataA[l + i * n];
			 }
		     Y[k+i*new_dim] += CosineCoef[j] * cos(x) + SineCoef[j] * sin(x);
         }
       }
     }
   }
   else
   {
      mexErrMsgTxt("Wrong number of input arguments.");
   }

   return;
}
