%/***************************************************************************
% *   Copyright (C) 2010 by Kamaledin Ghiasi-Shirazi                        *
% *   ghiathi@gmail.com                                                     *
% *                                                                         *
% *   This program is free software; you can redistribute it and/or modify  *
% *   it under the terms of the GNU General Public License as published by  *
% *   the Free Software Foundation; either version 3 of the License, or     *
% *   (at your option) any later version.                                   *
% *                                                                         *
% *   This program is distributed in the hope that it will be useful,       *
% *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
% *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
% *   GNU General Public License for more details.                          *
% *                                                                         *
% *   You should have received a copy of the GNU General Public License     *
% *   along with this program; if not, write to the                         *
% *   Free Software Foundation, Inc.,                                       *
% *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
% ***************************************************************************/

#include "mex.h"
#include "matrix.h"
#include <math.h>
#include <stdlib.h>

/* ==============================================================
 Main MEX function - interface to Matlab.
============================================================== */
void mexFunction( int nlhs, mxArray *plhs[],
		  int nrhs, const mxArray *prhs[] )
{
   long i, j, k, l, m;
   long nsv, new_dim, num_data, dim, num_kernel;
   double *Alpha;
   double *Y;
   double Y_l;
   double *X, *sv_X, *sigmaArg, *sigmaWeights, *dist_ij;
   int sigmaArgM, sigmaArgN, sigmaWeightsM, sigmaWeightsN;
   double dist;

   /* Y = kernelproj_mex(X, Alpha, b, sv_X, ker, arg ) */
   /* ------------------------------------------- */
   if( nrhs == 5)
   {
      /* data matrix [dim x num_data] */
      if( !mxIsNumeric(prhs[0]) || !mxIsDouble(prhs[0]) ||
        mxIsEmpty(prhs[0])    || mxIsComplex(prhs[0]) )
        mexErrMsgTxt("Input data must be a real matrix.");

      if( !mxIsNumeric(prhs[1]) || !mxIsDouble(prhs[1]) ||
        mxIsEmpty(prhs[1])    || mxIsComplex(prhs[1]) )
        mexErrMsgTxt("Support vector data must be a real matrix.");

      /* multipliers Alpha [nsv  x new_dim] */
      if( !mxIsNumeric(prhs[2]) || !mxIsDouble(prhs[2]) ||
        mxIsEmpty(prhs[2])    || mxIsComplex(prhs[2]) )
        mexErrMsgTxt("Input Alpha must be a real matrix.");

 
     /*  get pointer to arguments  */
     sigmaArg = mxGetPr(prhs[3]);
     sigmaArgM = mxGetM(prhs[3]);
     sigmaArgN = mxGetN(prhs[3]);

     /*  get pointer to arguments  */
     sigmaWeights = mxGetPr(prhs[4]);
     sigmaWeightsM = mxGetM(prhs[4]);
     sigmaWeightsN = mxGetN(prhs[4]);
     
     /* get pointer at input vectors */
     X     = mxGetPr(prhs[0]);
     sv_X  = mxGetPr(prhs[1]);
     Alpha = mxGetPr(prhs[2]);
     num_kernel = sigmaArgM * sigmaArgN;

     /* get data dimensions */
     dim = mxGetM(prhs[0]);
     num_data = mxGetN(prhs[0]);
     nsv = mxGetM(prhs[2]);
     new_dim = mxGetN(prhs[2]);

     /* creates output kernel matrix. */
     plhs[0] = mxCreateDoubleMatrix(new_dim,num_data,mxREAL);
     Y = mxGetPr(plhs[0]);

     dist_ij = new double[nsv];
     for( i = 0; i < num_data; i++ ) {

       for( k = 0; k < new_dim; k++) {
         Y[k+i*new_dim] = 0;
       }


       for( j = 0; j < nsv; j++ ) {
               dist_ij[j] = 0;
               for( k = 0; k < dim; k++) 
               {
                         dist = *(X+(i*dim)+k) - *(sv_X+(j*dim)+k);
                         dist_ij[j] += dist * dist;
               }
       }

        for( l = 0; l < num_kernel; l++ ) { 
                for( k = 0; k < new_dim; k++) {               
                Y_l = 0;               
                       for( j = 0; j < nsv; j++ ) {
                              Y_l += exp (-sigmaArg[l] * dist_ij[j]) * Alpha[j+k*nsv];
                         }
                Y[k+i*new_dim] += sigmaWeights[l] * Y_l;
                }               
       }
      }
     delete[] dist_ij;
   }
   else
   {
      mexErrMsgTxt("Wrong number of input arguments.");
   }

   return;
}
